"use strict";

exports.__esModule = true;
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _events = require("./utils/events");

var _getScrollWidth = _interopRequireDefault(require("./utils/getScrollWidth"));

var _clearSelection = _interopRequireDefault(require("./utils/clearSelection"));

var _styles = _interopRequireDefault(require("./styles"));

var _scrollTo = _interopRequireDefault(require("./modules/scrollTo"));

var _mouseWithout = _interopRequireDefault(require("./modules/mouse.without.window"));

var _generateStyle = _interopRequireDefault(require("./utils/generateStyle"));

var _generateID = _interopRequireDefault(require("./utils/generateID"));

var _is = require("./utils/is");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/**
 * This is min height for Scroll Bar.
 * If children content will be very big
 * Scroll Bar stay 20 pixels
 * */
const minHeightScrollBar = 20;
const REINIT_MS = 250;

class CustomScroll extends _react.Component {
  constructor(props) {
    super(props);
    ['scroll-area', 'scroll-area-holder', 'scrollBar', 'customScroll', 'customScrollHolder', 'customScrollFrame'].forEach(r => {
      this[`${r}Ref`] = /*#__PURE__*/(0, _react.createRef)();
    });
    this.nextWrapperHeight = 0;
    this.nextHolderHeight = 0;
    this.scrollID = (0, _generateID.default)();

    if (!CustomScroll.scrollWidth) {
      CustomScroll.scrollWidth = (0, _getScrollWidth.default)();
    }

    let scrollWidth = CustomScroll.scrollWidth;

    let _this = this; // If this is Safari / iPhone / iPad or other browser / device with scrollWidth === 0


    this.isZero = scrollWidth === 0;
    this.isVirtualized = (0, _is.isObject)(props.virtualized);

    if (this.isZero) {
      scrollWidth = 17;
    }

    this.reset = function () {
      _this.removeListeners();

      _this.blockSelection(true);
    };

    let className = (0, _is.isDefined)(props.className) ? props.className : 'react-customscroll';

    this.restScrollAfterResize = function () {
      _this.nextWrapperHeight = 0;
      _this.nextHolderHeight = 0;
    };

    this.timer = {};
    /**
     * Reinitialize scroll bar every 500 ms
     * */

    this.interval = setInterval(this.reinit.bind(this), REINIT_MS);
    this.state = {
      width: `calc(100% + ${scrollWidth}px)`,
      selection: true,
      scrollAreaShow: false,
      animate: props.animate || true,
      classes: {
        'base': className,
        'holder': `${className}-holder`,
        'frame': `${className}-frame`,
        'area': `${className}-scrollbar-area`,
        'area-holder': `${className}-scrollbar-holder`,
        'scroll-bar': `${className}-scrollbar`
      },
      virtualState: this.isVirtualized ? this.getScrollBarStyles(props.scrollTo || 0) : null,
      styles: {}
    };

    if ((0, _is.isClient)()) {
      if (!document.getElementById(this.scrollID)) {
        (0, _generateStyle.default)(`#${this.scrollID}::-webkit-scrollbar { opacity: 0 }
#${this.scrollID}::-webkit-scrollbar-track-piece { background-color: transparent }`, this.scrollID);
      }
    }
  }

  applyStyles() {
    let scrollWidth = CustomScroll.scrollWidth;
    let isRtl = false;

    if ((0, _is.isClient)()) {
      let direction = global.getComputedStyle(this.customScrollHolder).direction;
      isRtl = direction === 'rtl';
    }

    this.setState(Object.assign(this.state, {
      styles: (0, _styles.default)({
        virtualized: this.isVirtualized,
        isZero: this.isZero,
        originalScrollWidth: scrollWidth,
        scrollWidth: (0, _is.isDefined)(this.props.scrollWidth) ? this.props.scrollWidth : '6px',
        scrollAreaColor: (0, _is.isDefined)(this.props.scrollAreaColor) ? this.props.scrollAreaColor : '#494949',
        scrollBarRadius: (0, _is.isDefined)(this.props.scrollBarRadius) ? this.props.scrollBarRadius : '6px',
        scrollBarColor: (0, _is.isDefined)(this.props.scrollBarColor) ? this.props.scrollBarColor : '#aeaeae'
      }, isRtl)
    }));
  }

  componentDidMount() {
    /**
     * If mouse cursor gone outside window
     * Will trigger event 'mouseWithoutWindow'
     * And all listeners will remove
     * Content in scroll block will be selectable
     * */
    (0, _events.on)(document, ['mouseWithoutWindow'], this.reset);
    (0, _events.on)(window, ['resize'], this.restScrollAfterResize);
    this.scrollBlock = this.customScrollHolderRef.current;
    this.customScroll = this.customScrollRef.current;
    this.customScrollHolder = this.customScrollFrameRef.current;
    this.applyStyles();
  }

  componentWillUnmount() {
    if ((0, _is.isClient)()) {
      let el = document.getElementById(this.scrollID);

      if (el) {
        el.parentNode.removeChild(el);
      }
    }

    clearInterval(this.interval);
    this.removeListeners();
    clearTimeout(this.timer);
  }

  getParams() {
    let wrapperHeight = 0,
        holderHeight = 0,
        percentDiff = 0,
        height = 0;

    if (!(0, _is.isClient)()) {
      return {
        wrapperHeight,
        holderHeight,
        percentDiff,
        height
      };
    }

    let scrollArea = this['scroll-areaRef'].current;
    let paddings = window && scrollArea ? parseFloat(window.getComputedStyle(scrollArea, null).getPropertyValue('padding-top')) + parseFloat(window.getComputedStyle(scrollArea, null).getPropertyValue('padding-bottom')) : 0;

    if (this.isVirtualized) {
      wrapperHeight = this.props.virtualized.height || 0;
      holderHeight = this.props.virtualized.scrollHeight || 0;
    } else {
      wrapperHeight = this.customScroll && this.customScroll.offsetHeight;
      holderHeight = this.customScroll && this.customScrollHolder.offsetHeight;
    }

    if (holderHeight === 0) {
      height = 0;
      percentDiff = 0;
    } else {
      percentDiff = (wrapperHeight - paddings) / holderHeight;
      height = wrapperHeight * percentDiff;
    }

    return {
      wrapperHeight: Math.ceil(wrapperHeight),
      holderHeight,
      percentDiff,
      height
    };
  }

  blockSelection(state) {
    if (!state) {
      (0, _clearSelection.default)();
    }

    this.setState({
      selection: !!state
    });
  }

  onMouseDown(evt) {
    evt.stopPropagation();
    evt.preventDefault();
    /**
     * If we clicked right mouse button we must skip this event
     * */

    let isRightMB;

    if ('which' in evt) {
      isRightMB = evt.which === 3;
    } else if ('button' in evt) {
      isRightMB = evt.button === 2;
    }

    if (isRightMB) {
      setTimeout(this.endScroll);
      return false;
    }

    let elem = this.scrollBlock;
    let startPoint = evt.touches ? evt.touches[0].pageY : evt.pageY;
    let scrollTopOffset = this.isVirtualized ? this.props.scrollTo || 0 : elem.scrollTop;
    this.blockSelection(false);

    let _this = this;

    this.scrollRun = function (e) {
      e.stopPropagation();
      e.preventDefault();

      let {
        holderHeight,
        wrapperHeight
      } = _this.getParams();

      let diff = holderHeight / wrapperHeight;
      let pageY = e.touches ? e.touches[0].pageY : e.pageY;

      if (_this.isVirtualized) {
        let scrollTop = (pageY - startPoint) * diff + scrollTopOffset;
        scrollTop = holderHeight - wrapperHeight <= scrollTop ? holderHeight - wrapperHeight : scrollTop;

        if ((0, _is.isFunction)(_this.props.scrollSync)) {
          _this.props.scrollSync(scrollTop);
        }
      } else {
        (0, _scrollTo.default)(elem, (pageY - startPoint) * diff + scrollTopOffset);
      }
    };

    this.endScroll = function () {
      _this.removeListeners();

      _this.blockSelection(true);
    };

    (0, _events.on)(document, ['mouseup', 'touchend'], this.endScroll);
    (0, _events.on)(document, ['mousemove', 'touchmove'], this.scrollRun);
  }

  removeListeners() {
    (0, _events.off)(document, ['mouseWithoutWindow'], this.reset);
    (0, _events.off)(window, ['resize'], this.restScrollAfterResize);
    (0, _events.off)(document, ['mouseup', 'touchend'], this.endScroll);
    (0, _events.off)(document, ['mousemove', 'touchmove'], this.scrollRun);
  }

  reinit() {
    let {
      wrapperHeight,
      holderHeight
    } = this.getParams();

    if (wrapperHeight !== this.nextWrapperHeight || holderHeight !== this.nextHolderHeight) {
      if (this.isVirtualized) {
        let scrollPosition = this.props.scrollTo || 0;
        let virtualState = this.getScrollBarStyles(scrollPosition);
        this.setState({
          virtualState,
          scrollAreaShow: holderHeight > wrapperHeight
        });
      } else {
        this.setState({
          scrollAreaShow: holderHeight > wrapperHeight
        });
      }
    }

    this.nextWrapperHeight = wrapperHeight;
    this.nextHolderHeight = holderHeight;
  }

  jump(e) {
    let y = e.touches ? e.touches[0].pageY : e.pageY;
    let scrollBar = this.scrollBarRef.current;
    let scrollPosition = this.scrollBlock.scrollTop;
    let {
      wrapperHeight
    } = this.getParams();
    let topOffset = this.scrollBlock.getBoundingClientRect().top;

    if (this.isVirtualized) {
      scrollBar = {};
      scrollPosition = this.props.scrollTo || 0;
      scrollBar.offsetTop = this.state.virtualState.top;
      scrollBar.offsetHeight = this.state.virtualState.height;
    }

    if (y < topOffset + scrollBar.offsetTop || y > topOffset + scrollBar.offsetTop + scrollBar.offsetHeight) {
      let offset = topOffset + scrollBar.offsetTop <= y ? 1 : -1;
      let scrollY = scrollPosition + wrapperHeight * offset;

      if (this.isVirtualized) {
        if ((0, _is.isFunction)(this.props.scrollSync)) {
          this.props.scrollSync(scrollY);
        }
      } else {
        (0, _scrollTo.default)(this.scrollBlock, scrollY);
      }
    }
  }

  getScrollArea() {
    return /*#__PURE__*/_react.default.createElement("div", {
      ref: this['scroll-areaRef'],
      style: this.state.styles.scrollArea,
      onClick: this.jump.bind(this),
      className: this.state.classes.area
    }, /*#__PURE__*/_react.default.createElement("div", {
      ref: this['scroll-area-holderRef'],
      style: this.state.styles.scrollAreaFrame,
      className: this.state.classes['area-holder']
    }, /*#__PURE__*/_react.default.createElement("div", {
      ref: this['scrollBarRef'],
      style: Object.assign({}, this.state.styles.scrollBar, this.isVirtualized ? this.state.virtualState : this.getScrollBarStyles.call(this)),
      onMouseDown: this.onMouseDown.bind(this),
      onTouchStart: this.onMouseDown.bind(this),
      className: this.state.classes['scroll-bar']
    })));
  }

  scroll() {
    this.setState({
      scrollTop: this.scrollBlock.scrollTop
    });
  }

  getScrollBarStyles(offsetY) {
    let {
      holderHeight,
      percentDiff,
      height
    } = this.getParams();

    if (holderHeight === 0 && percentDiff === 0 && height === 0) {
      return {
        top: 0,
        height: 0
      };
    }

    let scrollTop = this.isVirtualized ? offsetY : this.state.scrollTop || this.scrollBlock.scrollTop;
    let newPercentDiff = height < minHeightScrollBar ? percentDiff - (minHeightScrollBar - height) / holderHeight : percentDiff;
    let scrollBarHeight = height < minHeightScrollBar ? minHeightScrollBar : height;
    return {
      top: scrollTop * newPercentDiff,
      height: scrollBarHeight
    };
  }

  componentDidUpdate(prevProps) {
    let offsetY = parseInt(this.props.scrollTo, 10);

    if ((0, _is.isDefined)(offsetY) && !isNaN(offsetY)) {
      if (prevProps.scrollTo !== offsetY) {
        if (this.isVirtualized) {
          offsetY = offsetY || 0;
          this.setState({
            virtualState: this.getScrollBarStyles(offsetY)
          });
        } else {
          _scrollTo.default.call(this, this.scrollBlock, offsetY, this.state.animate);
        }
      }
    }
  }

  setY(value) {
    _scrollTo.default.call(this, this.scrollBlock, value, this.state.animate);
  }

  render() {
    let ctmScroll = !this.state.selection ? Object.assign({}, this.state.styles.ctmScroll, this.state.styles.noselect) : this.state.styles.ctmScroll,
        ctmScrollFrame = this.state.scrollAreaShow ? Object.assign({}, this.state.styles.ctmScrollFrame, this.state.styles.ctmScrollActive) : this.state.styles.ctmScrollFrame;
    return /*#__PURE__*/_react.default.createElement("div", {
      ref: this['customScrollRef'],
      style: Object.assign({
        boxSizing: 'border-box'
      }, ctmScroll),
      className: this.state.classes.base,
      suppressHydrationWarning: true
    }, /*#__PURE__*/_react.default.createElement("div", {
      ref: this['customScrollHolderRef'],
      style: Object.assign({
        boxSizing: 'border-box'
      }, {
        width: this.state.width
      }, this.state.styles.ctmScrollHolder),
      onScroll: this.scroll.bind(this),
      className: this.state.classes.holder,
      id: this.scrollID,
      suppressHydrationWarning: true
    }, /*#__PURE__*/_react.default.createElement("div", {
      ref: this['customScrollFrameRef'],
      style: Object.assign({
        boxSizing: 'border-box'
      }, ctmScrollFrame, this.isZero ? {
        width: '100%'
      } : {}),
      className: this.state.classes.frame,
      suppressHydrationWarning: true
    }, (0, _is.isFunction)(this.props.children) ? this.props.children(this.scrollBlock && this.scrollBlock.scrollTop ? this.scrollBlock.scrollTop : 0) : this.props.children), this.state.scrollAreaShow ? this.getScrollArea.call(this) : null));
  }

}

var _default = CustomScroll;
exports.default = _default;